/*
  gravity for ikkeiplexing shield by ikkei
  http://blog.goo.ne.jp/jh3kxm
  
  
  History:
        2010-12-11 - V1.0 bounce by ikkei
        
*/

#include <ikkeiplexing.h>

ikkeiplexing iLedSign;

#define AVE 8     // averaging number of times
#define REF1 3    // Reflectance is REF1/REF2
#define REF2 4
#define MAG 1024  // Magnification

static uint8_t spt = 0;     // averaging pointer

const  Coordinate MAX = { 15, 7 };
static Coordinate previous = {1, 1}; // previous position
static Coordinate  present = {1, 1}; // present position
static Coordinate acceleration_in[AVE]; // averaging buffer
static Coordinate acceleration;
static Coordinate velocity;
static Coordinate position;
static Coordinate offset;


void gravity(){
  Coordinate sensor = iLedSign.GetAcceleration();
  acceleration_in[ spt ].x = sensor.x - offset.x;
  acceleration_in[ spt ].y = sensor.y - offset.y;

  spt++;
  if ( spt >= AVE ){
    spt = 0;
  }

  acceleration.x = 0;
  acceleration.y = 0;
  for (byte i=0; i < AVE; i++){
    acceleration.x +=  acceleration_in[ i ].x;
    acceleration.y +=  acceleration_in[ i ].y;
  }
  acceleration.x /= AVE;
  acceleration.y /= AVE;
 
  // integrating acceleration makes velocity
  velocity.x +=  acceleration.x;
  velocity.y += -acceleration.y;
  
  // integrating velocity makes position
  position.x += velocity.x;
  position.y += velocity.y;

  // 0 < x < MAX.x
  if ( position.x >= MAG * MAX.x){
    position.x = MAG * MAX.x - 1;
    velocity.x = -( velocity.x * REF1 ) / REF2;
  }else if ( position.x < MAG ){
    position.x = MAG;
    velocity.x = -( velocity.x * REF1 ) / REF2;
  }
  // 0 < y < MAX.y
  if ( position.y >= MAG * MAX.y){
    position.y = MAG * MAX.y - 1;
    velocity.y = -( velocity.y * REF1 ) / REF2;
  }else if ( position.y < MAG ){
    position.y = MAG;
    velocity.y = -( velocity.y * REF1 ) / REF2;
  }

  present.x = position.x / MAG;
  present.y = position.y / MAG;

  // display limitter
  constrain(present.x, 1, MAX.x - 1);
  constrain(present.y, 1, MAX.y - 1);

  // turn off the last position
  iLedSign.Set(previous.x,    previous.y,    0);
  iLedSign.Set(previous.x +1, previous.y,    0);
  iLedSign.Set(previous.x -1, previous.y,    0);
  iLedSign.Set(previous.x,    previous.y +1, 0);
  iLedSign.Set(previous.x,    previous.y -1, 0);
  
  // turn on the next position
  iLedSign.Set(present.x,    present.y,    1);
  iLedSign.Set(present.x +1, present.y,    1);
  iLedSign.Set(present.x -1, present.y,    1);
  iLedSign.Set(present.x,    present.y +1, 1);
  iLedSign.Set(present.x,    present.y -1, 1);
  
  // store last position
  previous.x = present.x;
  previous.y = present.y;
}

void setup(){
  iLedSign.Init(DOMINODUINO, SINGLE_BUFFER);
  iLedSign.Clear();
  delay(100);
  Coordinate sensor = iLedSign.GetAcceleration();
  offset.x = sensor.x;
  offset.y = sensor.y;
  position.x = 1;
  position.y = 1;
}

void loop(){
  if (iLedSign.GetPeriod(FRAME)){
    gravity();
  }
}


